/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * fn-japanese.c:  japanese function
 *
 * Author:
 *   Yukihiro Nakai (ynakai@redhat.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gnumeric-config.h>
#include <glib/gi18n.h>
#include <gnumeric.h>
#include <func.h>

#include <parse-util.h>
#include <cell.h>
#include <format.h>
#include <str.h>
#include <sheet.h>
#include <workbook.h>
#include <value.h>
#include <expr.h>
#include <mathfunc.h> /* This will change the round result... */

#include "plugin.h"
#include "plugin-util.h"
#include "module-plugin-defs.h"

#include "jisasc.h"

GNUMERIC_MODULE_PLUGIN_INFO_DECL;

/***************************************************************************/

static char const *help_jis = {
	N_("@FUNCTION=JIS\n"
	   "@SYNTAX=JIS(text)\n"

	   "@DESCRIPTION="
	   "JIS converts half-width chars in @text to full-width char "
	   "and return.\n"
	   "\n"
	   "* This function is Excel (Japanese version) "
           "compatible.\n"
	   "\n"
	   "@EXAMPLES=\n"
	   "\n"
	   "@SEEALSO=")
};

static GnmValue *
gnumeric_jis (FunctionEvalInfo *ei, GnmValue **argv)
{
	gchar *os;
	gchar* zen_utf8;

	os = g_strdup(value_peek_string(argv[0]));
	zen_utf8 = han2zen(os);
	g_free(os);
	return value_new_string_nocopy(zen_utf8);

}

static char const *help_asc = {
	N_("@FUNCTION=ASC\n"
	   "@SYNTAX=ASC(text)\n"

	   "@DESCRIPTION="
	   "ASC converts full-width chars in @text to half-width char "
	   "and return.\n"
	   "\n"
	   "* This function is Excel (Japanese version) "
           "compatible.\n"
	   "\n"
	   "@EXAMPLES=\n"
	   "\n"
	   "@SEEALSO=")
};

static GnmValue *
gnumeric_asc (FunctionEvalInfo *ei, GnmValue **argv)
{
	gchar *os;
	GnmValue* result;
	gchar* han_utf8;

	os = g_strdup(value_peek_string(argv[0]));
	han_utf8 = zen2han(os);
	result = value_new_string(han_utf8);

	g_free(os);
	g_free(han_utf8);

	return result;
}

static char const *help_yen = {
	N_("@FUNCTION=YEN\n"
	   "@SYNTAX=YEN(num[,order])\n"

	   "@DESCRIPTION="
	   "YEN returns @num formatted as currency in yen.\n"
	   "\n"
	   "* This function is Excel (Japanese version) "
           "compatible.\n"
	   "\n"
	   "@EXAMPLES=\n"
	   "\n"
	   "@SEEALSO=DOLLAR, FIXED")
};

/* This code is from fn-string/functions.c: gnumeric_dollar() */
static GnmValue *
gnumeric_yen (FunctionEvalInfo *ei, GnmValue **argv)
{
	gboolean precedes = TRUE, space_sep = FALSE;
	char const *curr = "￥";
	char *format, *s;
	GnmFormat *sf;
	char const *base_format =
		"%s#,##0%s%s;(%s#,##0%s)%s;_(%s\"-\"??%s_);_(@_)";
        gnm_float number = value_get_as_float (argv[0]);
	/* decimals default is 0, not 2 in gnumeric_dollar */
        int decimals = argv[1] ? value_get_as_int (argv[1]) : 0;
	gnm_float p10;
	GnmValue *v;
	char dotdecimals[1000];

	if (decimals > 0) {
		/* ".0000" */
		decimals = MIN (decimals, (int)sizeof (dotdecimals) - 2); /* FIXME? */
		dotdecimals[0] = '.';
		memset (&dotdecimals[1], '0', decimals);
		dotdecimals[decimals + 1] = 0;
	} else {
		dotdecimals[0] = 0;
	}

	if (precedes) {
		char *pre = g_strconcat ("\"", curr, "\"",
					 (space_sep ? " " : ""),
					 NULL);

		format = g_strdup_printf (base_format,
					  pre, dotdecimals, "",
					  pre, dotdecimals, "",
					  pre, "");
		g_free (pre);
	} else {
		char *post = g_strconcat ((space_sep ? " " : ""),
					  "\"", curr, "\"",
					  NULL);

		format = g_strdup_printf (base_format,
					  "", dotdecimals, post,
					  "", dotdecimals, post,
					  "", post);
		g_free (post);
	}
	sf = style_format_new_XL (format, FALSE);
	g_free (format);
	g_return_val_if_fail (sf != NULL,
			      value_new_error_NA (ei->pos));

	/* Since decimals can be negative, round the number.  */
	p10 = gpow10 (decimals);
	number = gnumeric_fake_round (number * p10) / p10;
	v = value_new_float (number);

	s = format_value (sf, v, NULL, -1,
		workbook_date_conv (ei->pos->sheet->workbook));
	value_release (v);

	style_format_unref (sf);

	return value_new_string_nocopy (s);
}

/***************************************************************************/

const GnmFuncDescriptor japanese_functions[] = {
	{ "jis", "s", N_("text"), &help_jis,
	  gnumeric_jis, NULL, NULL, NULL, NULL,
	  GNM_FUNC_SIMPLE,
	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
	{ "asc", "s", N_("text"), &help_asc,
	  gnumeric_asc, NULL, NULL, NULL, NULL,
	  GNM_FUNC_SIMPLE,
	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
	{ "yen", "f|f", N_("num,decimals"), &help_yen,
	  gnumeric_yen, NULL, NULL, NULL, NULL,
	  GNM_FUNC_SIMPLE,
	  GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
        {NULL}
};

void
plugin_init (void)
{
	convtable_init();
}

void
plugin_cleanup (void)
{
	convtable_cleanup();
}
